@@ -1,5 +1,6 @@
 import gzip
 import os
+import pathlib
 import shutil
 import subprocess
 import tempfile
@@ -143,7 +144,6 @@ class T(unittest.TestCase):
         """is_distro_package()."""
         self.assertRaises(ValueError, impl.is_distro_package, "nonexisting")
         self.assertTrue(impl.is_distro_package("bash"))
-        # no False test here, hard to come up with a generic one
 
     def test_get_architecture(self):
         """get_architecture()."""
@@ -246,7 +246,7 @@ bin/true                                                admin/superutils
                 impl.get_file_package("usr/bin/frob", True, cache_dir),
                 "frob-utils",
             )
-            cache_dir_files = os.listdir(cache_dir)
+            cache_dir_files = sorted(os.listdir(cache_dir))
             self.assertEqual(len(cache_dir_files), 3)
             self.assertEqual(
                 impl.get_file_package("/bo/gu/s", True, cache_dir), None
@@ -268,10 +268,9 @@ bin/true                                                admin/superutils
 
             # outdated cache, must refresh the cache and hit the invalid
             # mirror
-            if "updates" in cache_dir_files[0]:
-                cache_file = cache_dir_files[1]
-            else:
-                cache_file = cache_dir_files[0]
+            cache_file = [
+                name for name in cache_dir_files if "updates" not in name
+            ].pop()
             now = int(time.time())
             os.utime(os.path.join(cache_dir, cache_file), (now, now - 90000))
         finally:
@@ -482,7 +481,7 @@ usr/bin/frob                                            foo/frob
             if len(fields) == 7:
                 # pick first diversion we have
                 break
-        else:
+        else:  # pragma: no cover
             self.fail(
                 f"No non-local diversion found. dpkg-divert output: {output}"
             )
@@ -532,6 +531,14 @@ deb http://secondary.mirror tuxy extra
             SystemError, impl._get_primary_mirror_from_apt_sources, s
         )
 
+    def test_mirror_from_apt_sources_with_options(self) -> None:
+        """Test _get_primary_mirror_from_apt_sources() with options"""
+        sources = pathlib.Path(self.workdir) / "sources.list"
+        expected = "https://example.com/"
+        sources.write_text(f"deb [ arch=riscv64 ] {expected} suite component")
+        actual = impl._get_primary_mirror_from_apt_sources(str(sources))
+        self.assertEqual(actual, expected)
+
     def test_get_modified_conffiles(self):
         """get_modified_conffiles()"""
         # very shallow
@@ -548,6 +555,7 @@ deb http://secondary.mirror tuxy extra
         self.assertNotEqual(arch, "")
         self.assertNotIn("\n", arch)
 
+    @skip_if_command_is_missing("dpkg-architecture")
     def test_get_library_paths(self):
         """get_library_paths()."""
         paths = impl.get_library_paths()
