@@ -45,6 +45,17 @@ class T(unittest.TestCase):
             f" {int(elapsed_time)} seconds."
         )
 
+    @unittest.mock.patch("time.sleep")
+    def test_wait_for_proc_cmdline_failure(self, sleep_mock):
+        """Test wait_for_proc_cmdline() helper runs into timeout."""
+        open_mock = unittest.mock.mock_open(read_data="")
+        with unittest.mock.patch("builtins.open", open_mock):
+            with self.assertRaises(AssertionError):
+                self.wait_for_proc_cmdline(12345, 0.3)
+        open_mock.assert_called_with("/proc/12345/cmdline", encoding="utf-8")
+        sleep_mock.assert_called_with(0.1)
+        self.assertEqual(sleep_mock.call_count, 3)
+
     def test_add_package_info(self):
         """add_package_info()."""
         # determine bash version
@@ -482,19 +493,19 @@ class T(unittest.TestCase):
             pr = apport.report.Report()
             pr["ExecutablePath"] = "/usr/bin/python3"
             pr["ProcStatus"] = "Name:\tpython3"
-            pr["ProcCmdline"] = "python\0-m\0re\0install"
+            pr["ProcCmdline"] = "python3\0-m\0re\0install"
             pr._check_interpreted()
             self.assertEqual(pr["InterpreterPath"], "/usr/bin/python3")
-            self.assertIn("re.py", pr["ExecutablePath"])
+            self.assertRegex(pr["ExecutablePath"], r"re(\.py|/__init__.py)$")
 
             # python script through -m, with dot separator; sub-level module
             pr = apport.report.Report()
             pr["ExecutablePath"] = "/usr/bin/python3"
             pr["ProcStatus"] = "Name:\tpython3"
-            pr["ProcCmdline"] = "python\0-m\0distutils.cmd\0foo"
+            pr["ProcCmdline"] = "python3\0-m\0http.server\08080"
             pr._check_interpreted()
             self.assertEqual(pr["InterpreterPath"], "/usr/bin/python3")
-            self.assertIn("distutils/cmd.py", pr["ExecutablePath"])
+            self.assertIn("http/server.py", pr["ExecutablePath"])
 
             # Python script through -m, non-existent module
             pr = apport.report.Report()
@@ -602,9 +613,8 @@ class T(unittest.TestCase):
         self.assertIn("UnreportableReason", pr)
         self.assertEqual(pr["InterpreterPath"], "/usr/bin/twistd")
 
-    @classmethod
     def _generate_sigsegv_report(
-        cls,
+        self,
         file=None,
         signal="11",
         code="""
@@ -638,12 +648,15 @@ int main() { return f(42); }
             # create a test executable
             with open("crash.c", "w", encoding="utf-8") as fd:
                 fd.write(code)
-            assert (
-                subprocess.call(
-                    ["gcc"] + extra_gcc_args + ["-g", "crash.c", "-o", "crash"]
+            try:
+                subprocess.run(
+                    ["gcc"]
+                    + extra_gcc_args
+                    + ["-g", "crash.c", "-o", "crash"],
+                    check=True,
                 )
-                == 0
-            )
+            except FileNotFoundError as error:
+                self.skipTest(f"{error.filename} not available")
             assert os.path.exists("crash")
 
             # call it through gdb and dump core
@@ -663,7 +676,7 @@ int main() { return f(42); }
                 env={"HOME": workdir},
                 stdout=subprocess.PIPE,
             )
-            cls._validate_core("core")
+            self._validate_core("core")
 
             pr["ExecutablePath"] = os.path.join(workdir, "crash")
             pr["CoreDump"] = (os.path.join(workdir, "core"),)
@@ -838,26 +851,26 @@ int main() { return f(42); }
             with open(rep.name, "rb") as f:
                 pr.load(f)
             pr["Signal"] = "1"
-            pr.add_hooks_info("fake_ui")
+            pr.add_hooks_info()
             self.assertNotIn("SegvAnalysis", pr)
 
             pr = apport.report.Report()
             with open(rep.name, "rb") as f:
                 pr.load(f)
-        pr.add_hooks_info("fake_ui")
+        pr.add_hooks_info()
         self.assertIn(
             'Skipped: missing required field "Architecture"',
             pr["SegvAnalysis"],
         )
 
         pr.add_os_info()
-        pr.add_hooks_info("fake_ui")
+        pr.add_hooks_info()
         self.assertIn(
             'Skipped: missing required field "ProcMaps"', pr["SegvAnalysis"]
         )
 
         pr.add_proc_info()
-        pr.add_hooks_info("fake_ui")
+        pr.add_hooks_info()
         if pr["Architecture"] in ["amd64", "i386"]:
             # data/general-hooks/parse_segv.py only runs for x86 and x86_64
             self.assertIn(
@@ -1345,7 +1358,7 @@ int main() { return f(42); }
             r = apport.report.Report()
             r["Package"] = "bar"
             # should not throw any exceptions
-            self.assertEqual(r.add_hooks_info("fake_ui"), False)
+            self.assertEqual(r.add_hooks_info(), False)
             self.assertEqual(
                 set(r.keys()),
                 set(
@@ -1364,7 +1377,7 @@ int main() { return f(42); }
             r = apport.report.Report()
             r["Package"] = "baz 1.2-3"
             # should not throw any exceptions
-            self.assertEqual(r.add_hooks_info("fake_ui"), False)
+            self.assertEqual(r.add_hooks_info(), False)
             self.assertEqual(
                 set(r.keys()),
                 set(
@@ -1382,7 +1395,7 @@ int main() { return f(42); }
 
             r = apport.report.Report()
             r["Package"] = "foo"
-            self.assertEqual(r.add_hooks_info("fake_ui"), False)
+            self.assertEqual(r.add_hooks_info(), False)
             self.assertEqual(
                 set(r.keys()),
                 set(
@@ -1403,11 +1416,11 @@ int main() { return f(42); }
             self.assertEqual(r["Field2"], "Field 2\nBla")
             self.assertEqual(r["CommonField1"], "CommonField 1")
             self.assertEqual(r["CommonField2"], "CommonField 2")
-            self.assertEqual(r["CommonField3"], "fake_ui")
+            self.assertEqual(r["CommonField3"], "NoninteractiveHookUI()")
 
             r = apport.report.Report()
             r["Package"] = "foo 4.5-6"
-            self.assertEqual(r.add_hooks_info("fake_ui"), False)
+            self.assertEqual(r.add_hooks_info(), False)
             self.assertEqual(
                 set(r.keys()),
                 set(
@@ -1431,10 +1444,10 @@ int main() { return f(42); }
 
             # test hook abort
             r["Spethial"] = "1"
-            self.assertEqual(r.add_hooks_info("fake_ui"), True)
+            self.assertEqual(r.add_hooks_info(), True)
             r = apport.report.Report()
             r["Package"] = "commonspethial"
-            self.assertEqual(r.add_hooks_info("fake_ui"), True)
+            self.assertEqual(r.add_hooks_info(), True)
 
             # source package hook
             with open(
@@ -1456,7 +1469,7 @@ int main() { return f(42); }
             r = apport.report.Report()
             r["SourcePackage"] = "foo"
             r["Package"] = "libfoo 3"
-            self.assertEqual(r.add_hooks_info("fake_ui"), False)
+            self.assertEqual(r.add_hooks_info(), False)
             self.assertEqual(
                 set(r.keys()),
                 set(
@@ -1478,11 +1491,11 @@ int main() { return f(42); }
             self.assertEqual(r["Field2"], "Field 2\nBla")
             self.assertEqual(r["CommonField1"], "CommonField 1")
             self.assertEqual(r["CommonField2"], "CommonField 2")
-            self.assertEqual(r["CommonField3"], "fake_ui")
+            self.assertEqual(r["CommonField3"], "NoninteractiveHookUI()")
 
             # test hook abort
             r["Package"] = "spethial"
-            self.assertEqual(r.add_hooks_info("fake_ui"), True)
+            self.assertEqual(r.add_hooks_info(), True)
 
         finally:
             shutil.rmtree(apport.report.GENERAL_HOOK_DIR)
@@ -1540,7 +1553,7 @@ int main() { return f(42); }
                 "%s/foolabs.example.com/foo/bin/frob" % apport.report._opt_dir
             )
 
-            self.assertEqual(r.add_hooks_info("fake_ui"), False)
+            self.assertEqual(r.add_hooks_info(), False)
             self.assertEqual(r["SourceHook"], "1")
         finally:
             shutil.rmtree(apport.report.GENERAL_HOOK_DIR)
@@ -1594,7 +1607,7 @@ int main() { return f(42); }
             r["SourcePackage"] = "foo"
             r["ExecutablePath"] = "/bin/foo-cli"
 
-            self.assertEqual(r.add_hooks_info("fake_ui"), False)
+            self.assertEqual(r.add_hooks_info(), False)
 
             # should have the data until the crash
             self.assertEqual(r["BinHookBefore"], "1")
@@ -1690,10 +1703,10 @@ int main() { return f(42); }
             shutil.rmtree(workdir)
             apport.report.apport.report._ignore_file = orig_ignore_file
 
-    def test_blacklisting(self):
-        """check_ignored() for system-wise blacklist."""
-        orig_blacklist_dir = apport.report._blacklist_dir
-        apport.report._blacklist_dir = tempfile.mkdtemp()
+    def test_denylisting(self):
+        """check_ignored() for system-wise denylist."""
+        orig_denylist_dir = apport.report._DENYLIST_DIR
+        apport.report._DENYLIST_DIR = tempfile.mkdtemp()
         orig_ignore_file = apport.report._ignore_file
         apport.report._ignore_file = "/nonexistant"
         try:
@@ -1708,7 +1721,7 @@ int main() { return f(42); }
 
             # should not stumble over comments
             with open(
-                os.path.join(apport.report._blacklist_dir, "README"),
+                os.path.join(apport.report._DENYLIST_DIR, "README"),
                 "w",
                 encoding="utf-8",
             ) as fd:
@@ -1716,7 +1729,7 @@ int main() { return f(42); }
 
             # no ignores on nonmatching paths
             with open(
-                os.path.join(apport.report._blacklist_dir, "bl1"),
+                os.path.join(apport.report._DENYLIST_DIR, "bl1"),
                 "w",
                 encoding="utf-8",
             ) as fd:
@@ -1726,7 +1739,7 @@ int main() { return f(42); }
 
             # ignore crap now
             with open(
-                os.path.join(apport.report._blacklist_dir, "bl_2"),
+                os.path.join(apport.report._DENYLIST_DIR, "bl_2"),
                 "w",
                 encoding="utf-8",
             ) as fd:
@@ -1736,7 +1749,7 @@ int main() { return f(42); }
 
             # ignore bash now
             with open(
-                os.path.join(apport.report._blacklist_dir, "bl1"),
+                os.path.join(apport.report._DENYLIST_DIR, "bl1"),
                 "a",
                 encoding="utf-8",
             ) as fd:
@@ -1744,14 +1757,14 @@ int main() { return f(42); }
             self.assertEqual(bash_rep.check_ignored(), True)
             self.assertEqual(crap_rep.check_ignored(), True)
         finally:
-            shutil.rmtree(apport.report._blacklist_dir)
-            apport.report._blacklist_dir = orig_blacklist_dir
+            shutil.rmtree(apport.report._DENYLIST_DIR)
+            apport.report._DENYLIST_DIR = orig_denylist_dir
             apport.report._ignore_file = orig_ignore_file
 
-    def test_whitelisting(self):
-        """check_ignored() for system-wise whitelist."""
-        orig_whitelist_dir = apport.report._whitelist_dir
-        apport.report._whitelist_dir = tempfile.mkdtemp()
+    def test_allowlisting(self):
+        """check_ignored() for system-wise allowlist."""
+        orig_allowlist_dir = apport.report._ALLOWLIST_DIR
+        apport.report._ALLOWLIST_DIR = tempfile.mkdtemp()
         orig_ignore_file = apport.report.apport.report._ignore_file
         apport.report.apport.report._ignore_file = "/nonexistant"
         try:
@@ -1760,13 +1773,13 @@ int main() { return f(42); }
             crap_rep = apport.report.Report()
             crap_rep["ExecutablePath"] = "/bin/crap"
 
-            # no ignores without any whitelist
+            # no ignores without any allowlist
             self.assertEqual(bash_rep.check_ignored(), False)
             self.assertEqual(crap_rep.check_ignored(), False)
 
             # should not stumble over comments
             with open(
-                os.path.join(apport.report._whitelist_dir, "README"),
+                os.path.join(apport.report._ALLOWLIST_DIR, "README"),
                 "w",
                 encoding="utf-8",
             ) as fd:
@@ -1774,7 +1787,7 @@ int main() { return f(42); }
 
             # accepts matching paths
             with open(
-                os.path.join(apport.report._whitelist_dir, "wl1"),
+                os.path.join(apport.report._ALLOWLIST_DIR, "wl1"),
                 "w",
                 encoding="utf-8",
             ) as fd:
@@ -1784,7 +1797,7 @@ int main() { return f(42); }
 
             # also accept crap now
             with open(
-                os.path.join(apport.report._whitelist_dir, "wl_2"),
+                os.path.join(apport.report._ALLOWLIST_DIR, "wl_2"),
                 "w",
                 encoding="utf-8",
             ) as fd:
@@ -1794,7 +1807,7 @@ int main() { return f(42); }
 
             # only complete matches accepted
             with open(
-                os.path.join(apport.report._whitelist_dir, "wl1"),
+                os.path.join(apport.report._ALLOWLIST_DIR, "wl1"),
                 "w",
                 encoding="utf-8",
             ) as fd:
@@ -1802,8 +1815,8 @@ int main() { return f(42); }
             self.assertEqual(bash_rep.check_ignored(), True)
             self.assertEqual(crap_rep.check_ignored(), False)
         finally:
-            shutil.rmtree(apport.report._whitelist_dir)
-            apport.report._whitelist_dir = orig_whitelist_dir
+            shutil.rmtree(apport.report._ALLOWLIST_DIR)
+            apport.report._ALLOWLIST_DIR = orig_allowlist_dir
             apport.report.apport.report._ignore_file = orig_ignore_file
 
     def test_obsolete_packages(self):
@@ -1893,10 +1906,18 @@ int main() { return f(42); }
             timestamp, time.mktime(time.strptime("2014-01-01", "%Y-%m-%d"))
         )
 
+    def test_command_output(self):
+        out = apport.report._command_output(["echo", "hello"])
+        self.assertEqual(out, "hello")
+
     def test_command_output_passes_env(self):
         fake_env = {"GCONV_PATH": "/tmp"}
         out = apport.report._command_output(["env"], env=fake_env)
-        self.assertIn(b"GCONV_PATH", out)
+        self.assertIn("GCONV_PATH", out)
+
+    def test_command_output_raises_error(self):
+        with self.assertRaisesRegex(OSError, "failed with exit code 1"):
+            apport.report._command_output(["false"])
 
     def test_extrapath_preferred(self):
         """If extrapath is passed it is preferred."""
