@@ -7,7 +7,8 @@
 # option) any later version.  See http://www.gnu.org/copyleft/gpl.html for
 # the full text of the license.
 
-import array
+import argparse
+import collections
 import grp
 import os
 import resource
@@ -26,9 +27,12 @@ import unittest
 import psutil
 
 import apport.fileutils
-from tests.helper import read_shebang
+from tests.helper import import_module_from_file, read_shebang
 from tests.paths import get_data_directory, local_test_environment
 
+APPORT_PATH = get_data_directory() / "apport"
+apport_binary = import_module_from_file(APPORT_PATH)
+
 test_package = "coreutils"
 test_source = "coreutils"
 
@@ -58,8 +62,6 @@ class T(unittest.TestCase):
         cls.orig_environ = os.environ.copy()
         os.environ |= local_test_environment()
 
-        cls.apport_path = os.path.join(get_data_directory(), "apport")
-
         cls.orig_cwd = os.getcwd()
         cls.orig_core_dir = apport.fileutils.core_dir
         cls.orig_ignore_file = apport.report._ignore_file
@@ -120,7 +122,7 @@ class T(unittest.TestCase):
         try:
             with subprocess.Popen(
                 [
-                    self.apport_path,
+                    str(APPORT_PATH),
                     "-p",
                     str(test_proc.pid),
                     "-s",
@@ -218,7 +220,7 @@ class T(unittest.TestCase):
         try:
             with subprocess.Popen(
                 [
-                    self.apport_path,
+                    str(APPORT_PATH),
                     "-p",
                     str(test_proc.pid),
                     "-s",
@@ -231,12 +233,11 @@ class T(unittest.TestCase):
                 stdin=subprocess.PIPE,
                 stderr=subprocess.PIPE,
             ) as app:
-
                 time.sleep(0.5)  # give it some time to grab the lock
 
                 with subprocess.Popen(
                     [
-                        self.apport_path,
+                        str(APPORT_PATH),
                         "-p",
                         str(test_proc2.pid),
                         "-s",
@@ -249,7 +250,6 @@ class T(unittest.TestCase):
                     stdin=subprocess.PIPE,
                     stderr=subprocess.PIPE,
                 ) as app2:
-
                     # app should wait indefinitely for stdin, while app2 should
                     # terminate immediately (give it 5 seconds)
                     timeout = 50
@@ -322,7 +322,7 @@ class T(unittest.TestCase):
         env = os.environ.copy()
         env["APPORT_LOG_FILE"] = log
         app = subprocess.run(
-            [self.apport_path],
+            [str(APPORT_PATH)],
             check=False,
             env=env,
             preexec_fn=close_stdin_and_stderr,
@@ -338,6 +338,10 @@ class T(unittest.TestCase):
         """Apport ignores SIGQUIT."""
         self.do_crash(sig=signal.SIGQUIT, expect_report=False)
 
+    def test_ignore_sigxcpu(self):
+        """Apport ignores CPU time limit exceeded (SIGXCPU)."""
+        self.do_crash(sig=signal.SIGXCPU, expect_report=False)
+
     def test_leak_inaccessible_files(self):
         """Existence of user-inaccessible files do not leak."""
         local_exe = os.path.join(self.workdir, "myscript")
@@ -408,7 +412,7 @@ class T(unittest.TestCase):
         """Packaged executables create core dumps on proper ulimits."""
         # for SEGV and ABRT we expect reports and core files
         for sig in (signal.SIGSEGV, signal.SIGABRT):
-            for (kb, exp_file) in core_ulimit_table:
+            for kb, exp_file in core_ulimit_table:
                 resource.setrlimit(resource.RLIMIT_CORE, (kb, -1))
                 self.do_crash(
                     expect_corefile=exp_file,
@@ -438,7 +442,7 @@ class T(unittest.TestCase):
         os.chmod(local_exe, 0o755)
 
         for sig in (signal.SIGSEGV, signal.SIGABRT, signal.SIGQUIT):
-            for (kb, exp_file) in core_ulimit_table:
+            for kb, exp_file in core_ulimit_table:
                 resource.setrlimit(resource.RLIMIT_CORE, (kb, -1))
                 self.do_crash(
                     expect_corefile=exp_file,
@@ -537,7 +541,7 @@ class T(unittest.TestCase):
 
             app = subprocess.run(
                 [
-                    self.apport_path,
+                    str(APPORT_PATH),
                     "-p",
                     str(test_proc.pid),
                     "-s",
@@ -578,7 +582,7 @@ class T(unittest.TestCase):
             env["APPORT_LOG_FILE"] = log
             app = subprocess.run(
                 [
-                    self.apport_path,
+                    str(APPORT_PATH),
                     "-p",
                     str(test_proc.pid),
                     "-s",
@@ -599,13 +603,14 @@ class T(unittest.TestCase):
             test_proc.kill()
             test_proc.wait()
 
-        if app.stdout != "" or app.stderr != "":
-            self.fail(
-                f"Apport wrote to stdout and/or stderr"
-                f" (exit code {app.returncode})."
-                f"\n*** stdout:\n{app.stdout.strip()}"
-                f"\n*** stderr:\n{app.stderr.strip()}"
-            )
+        self.assertEqual(
+            app.stdout + app.stderr,
+            "",
+            msg=f"Apport wrote to stdout and/or stderr"
+            f" (exit code {app.returncode})."
+            f"\n*** stdout:\n{app.stdout.strip()}"
+            f"\n*** stderr:\n{app.stderr.strip()}",
+        )
         self.assertEqual(app.returncode, 0, app.stderr)
         with open(log, encoding="utf-8") as f:
             logged = f.read()
@@ -630,7 +635,7 @@ class T(unittest.TestCase):
             env["APPORT_LOG_FILE"] = "/not/existing/apport.log"
             app = subprocess.run(
                 [
-                    self.apport_path,
+                    str(APPORT_PATH),
                     "-p",
                     str(test_proc.pid),
                     "-s",
@@ -782,15 +787,11 @@ class T(unittest.TestCase):
     # Helper methods
     #
 
-    def _call_apport(
-        self,
-        process: psutil.Process,
-        sig: int,
-        dump_mode: int,
-        stdin: typing.IO,
-    ) -> None:
-        cmd = [
-            self.apport_path,
+    @staticmethod
+    def _apport_args(
+        process: psutil.Process, sig: int, dump_mode: int
+    ) -> list[str]:
+        return [
             f"-p{process.pid}",
             f"-s{sig}",
             f"-c{resource.getrlimit(resource.RLIMIT_CORE)[0]}",
@@ -801,8 +802,34 @@ class T(unittest.TestCase):
             "--",
             process.exe().replace("/", "!"),
         ]
+
+    def _call_apport(
+        self,
+        process: psutil.Process,
+        sig: int,
+        dump_mode: int,
+        stdin: typing.IO,
+    ) -> None:
+        cmd = [str(APPORT_PATH)] + self._apport_args(process, sig, dump_mode)
         subprocess.check_call(cmd, stdin=stdin)
 
+    @staticmethod
+    def _forward_crash_to_container(
+        socket_path: str, args: argparse.Namespace, coredump_fd: int
+    ) -> None:
+        orig_os_open = os.open
+
+        def _mocked_os_open(
+            path, flags: int, dir_fd: typing.Optional[int] = None
+        ) -> int:
+            if path == "root/run/apport.socket":
+                return orig_os_open(socket_path, flags)
+            return orig_os_open(path, flags, dir_fd=dir_fd)
+
+        with unittest.mock.patch("os.open") as os_open_mock:
+            os_open_mock.side_effect = _mocked_os_open
+            apport_binary.forward_crash_to_container(args, coredump_fd, False)
+
     def _call_apport_via_socket(
         self,
         process: psutil.Process,
@@ -818,18 +845,15 @@ class T(unittest.TestCase):
         server.bind(socket_path)
         server.listen(1)
 
-        if os.fork() == 0:
-            client = socket.socket(socket.AF_UNIX, socket.SOCK_STREAM)
-            client.connect(socket_path)
-            core_ulimit = resource.getrlimit(resource.RLIMIT_CORE)[0]
-            args = f"{process.pid} {sig} {core_ulimit} {dump_mode}"
-            fd_msg = (
-                socket.SOL_SOCKET,
-                socket.SCM_RIGHTS,
-                array.array("i", [stdin.fileno()]),
-            )
-            client.sendmsg([args.encode()], [fd_msg])
-            os._exit(0)
+        args = apport_binary.parse_arguments(
+            self._apport_args(process, sig, dump_mode)
+        )
+        with unittest.mock.patch(
+            "apport.fileutils.search_map"
+        ) as search_map_mock:
+            search_map_mock.return_value = True
+            self._forward_crash_to_container(socket_path, args, stdin.fileno())
+            search_map_mock.assert_called()
 
         # call apport like systemd does via socket activation
         def child_setup():
@@ -843,7 +867,7 @@ class T(unittest.TestCase):
 
         try:
             subprocess.run(
-                [self.apport_path],
+                [str(APPORT_PATH)],
                 check=True,
                 preexec_fn=child_setup,
                 pass_fds=[3],
@@ -1148,6 +1172,34 @@ class T(unittest.TestCase):
                 f"within {int(timeout)} seconds."
             )
 
+    @unittest.mock.patch("os.path.exists")
+    @unittest.mock.patch("time.sleep")
+    def test_wait_for_core_file_core_not_created(
+        self, sleep_mock, exists_mock
+    ):
+        """Test wait_for_core_file() helper runs into timeout for core file."""
+        exists_mock.return_value = False
+        with self.assertRaises(AssertionError):
+            self.wait_for_core_file(123456789, "core")
+        sleep_mock.assert_called_with(0.1)
+        self.assertEqual(sleep_mock.call_count, 51)
+
+    @unittest.mock.patch("os.path.exists")
+    @unittest.mock.patch("psutil.Process", spec=psutil.Process)
+    @unittest.mock.patch("time.sleep")
+    def test_wait_for_core_file_timeout(
+        self, sleep_mock, process_mock, exists_mock
+    ):
+        """Test wait_for_core_file() helper runs into timeout."""
+        popenfile = collections.namedtuple("popenfile", ["path"])
+        exists_mock.return_value = True
+        process_mock.return_value.open_files.return_value = [popenfile("core")]
+        with unittest.mock.patch.object(self, "fail") as fail_mock:
+            self.wait_for_core_file(123456789, "core")
+        fail_mock.assert_called_once()
+        sleep_mock.assert_called_with(0.1)
+        self.assertEqual(sleep_mock.call_count, 600)
+
     def wait_for_gdb_child_process(
         self, gdb_pid: int, command: str
     ) -> psutil.Process:
@@ -1173,3 +1225,34 @@ class T(unittest.TestCase):
             f"GDB child process {command} not started within "
             f"{int(timeout)} seconds. GDB children: {gdb_children!r}"
         )
+
+    @unittest.mock.patch("time.sleep")
+    def test_wait_for_gdb_child_process(self, sleep_mock):
+        """Test wait_for_gdb_child_process() helper method."""
+        child = unittest.mock.MagicMock(spec=psutil.Process)
+        child.status.side_effect = ["tracing-stop", "sleeping"]
+        child.cmdline.return_value = [self.TEST_EXECUTABLE] + self.TEST_ARGS
+        with unittest.mock.patch(
+            "psutil.Process", spec=psutil.Process
+        ) as process_mock:
+            process_mock.return_value.children.side_effect = [
+                [],
+                [child],  # child not started (tracing-stop)
+                [child],  # child ready
+            ]
+            self.wait_for_gdb_child_process(123456789, self.TEST_EXECUTABLE)
+        sleep_mock.assert_called_with(0.1)
+        self.assertEqual(sleep_mock.call_count, 2)
+
+    @unittest.mock.patch("psutil.Process", spec=psutil.Process)
+    @unittest.mock.patch("time.sleep")
+    def test_wait_for_gdb_child_process_timeout(
+        self, sleep_mock, process_mock
+    ):
+        """Test wait_for_gdb_child_process() helper runs into timeout."""
+        process_mock.return_value.children.return_value = []
+        with unittest.mock.patch.object(self, "fail") as fail_mock:
+            self.wait_for_gdb_child_process(123456789, self.TEST_EXECUTABLE)
+        fail_mock.assert_called_once()
+        sleep_mock.assert_called_with(0.1)
+        self.assertEqual(sleep_mock.call_count, 51)
