@@ -16,37 +16,39 @@ import tempfile
 import textwrap
 import unittest
 import unittest.mock
-
-import gi
-
-gi.require_version("Gtk", "3.0")  # noqa: E402, pylint: disable=C0413
-from gi.repository import GLib, GObject, Gtk
+from gettext import gettext as _
 
 import apport.crashdb_impl.memory
 import apport.report
-from apport import unicode_gettext as _
 from tests.helper import import_module_from_file
-from tests.paths import is_local_source_directory, local_test_environment
+from tests.paths import get_data_directory, local_test_environment
 
-GLib.log_set_always_fatal(
-    GLib.LogLevelFlags.LEVEL_WARNING | GLib.LogLevelFlags.LEVEL_CRITICAL
-)
+try:
+    import gi
 
+    gi.require_version("Gtk", "3.0")  # noqa: E402, pylint: disable=C0413
+    from gi.repository import GLib, GObject, Gtk
 
-if is_local_source_directory():
-    apport_gtk_path = "gtk/apport-gtk"
-    kernel_oops_path = "data/kernel_oops"
+    GLib.log_set_always_fatal(
+        GLib.LogLevelFlags.LEVEL_WARNING | GLib.LogLevelFlags.LEVEL_CRITICAL
+    )
+
+    GI_IMPORT_ERROR = None
+except ImportError as error:
+    GI_IMPORT_ERROR = error
+
+apport_gtk_path = get_data_directory("gtk") / "apport-gtk"
+if not GI_IMPORT_ERROR:
+    apport_gtk = import_module_from_file(apport_gtk_path)
+    GTKUserInterface = apport_gtk.GTKUserInterface
 else:
-    apport_gtk_path = os.path.join(
-        os.environ.get("APPORT_DATA_DIR", "/usr/share/apport"), "apport-gtk"
-    )
-    kernel_oops_path = os.path.join(
-        os.environ.get("APPORT_DATA_DIR", "/usr/share/apport"), "kernel_oops"
-    )
-apport_gtk = import_module_from_file(apport_gtk_path)
-GTKUserInterface = apport_gtk.GTKUserInterface
+    apport_gtk = None
+    GTKUserInterface = None
 
 
+@unittest.skipIf(
+    GI_IMPORT_ERROR, f"gi Python module not available: {GI_IMPORT_ERROR}"
+)
 class T(unittest.TestCase):
     POLLING_INTERVAL_MS = 10
 
@@ -72,7 +74,7 @@ class T(unittest.TestCase):
         os.environ["APPORT_IGNORE_OBSOLETE_PACKAGES"] = "1"
         os.environ["APPORT_DISABLE_DISTRO_CHECK"] = "1"
 
-        self.app = GTKUserInterface([apport_gtk_path])
+        self.app = GTKUserInterface([str(apport_gtk_path)])
 
         # use in-memory crashdb
         self.app.crashdb = apport.crashdb_impl.memory.CrashDatabase(None, {})
@@ -107,7 +109,7 @@ class T(unittest.TestCase):
 
         GLib.idle_add(c)
         result = self.app.ui_present_report_details(True)
-        self.assertFalse(result["report"])
+        self.assertFalse(result.report)
 
     def test_kernel_crash_layout(self):
         """Display crash dialog for kernel crash.
@@ -584,7 +586,7 @@ class T(unittest.TestCase):
         self.app.can_examine_locally.return_value = True
         result = self.app.ui_present_report_details(True)
         self.assertTrue(self.app.w("examine").get_property("visible"))
-        self.assertTrue(result["examine"])
+        self.assertTrue(result.examine)
 
     def test_apport_bug_package_layout(self):
         """Display report detail dialog.
@@ -732,9 +734,9 @@ class T(unittest.TestCase):
 
         def cont():
             if Gtk.events_pending():
-                return True
+                return True  # pragma: no cover
             if not self.app.w("continue_button").get_visible():
-                return True
+                return True  # pragma: no cover
             self.app.w("continue_button").clicked()
             return False
 
@@ -786,19 +788,19 @@ class T(unittest.TestCase):
 
         def show_details():
             if Gtk.events_pending():
-                return True
+                return True  # pragma: no cover
             if not self.app.w("show_details").get_visible():
-                return True
+                return True  # pragma: no cover
             self.app.w("show_details").clicked()
             GLib.timeout_add(self.POLLING_INTERVAL_MS, cont)
             return False
 
         def cont():
             if Gtk.events_pending():
-                return True
+                return True  # pragma: no cover
             # wait until data collection is done and tree filled
             if self.app.tree_model.get_iter_first() is None:
-                return True
+                return True  # pragma: no cover
 
             self.assertTrue(self.app.w("continue_button").get_visible())
             self.app.w("continue_button").clicked()
@@ -852,9 +854,9 @@ class T(unittest.TestCase):
 
         def show_details():
             if Gtk.events_pending():
-                return True
+                return True  # pragma: no cover
             if not self.app.w("show_details").get_visible():
-                return True
+                return True  # pragma: no cover
             self.app.w("show_details").clicked()
             GLib.timeout_add(self.POLLING_INTERVAL_MS, cont)
             return False
@@ -862,9 +864,9 @@ class T(unittest.TestCase):
         def cont():
             # wait until data collection is done and tree filled
             if Gtk.events_pending():
-                return True
+                return True  # pragma: no cover
             if self.app.tree_model.get_iter_first() is None:
-                return True
+                return True  # pragma: no cover
 
             self.assertTrue(self.app.w("continue_button").get_visible())
             self.app.w("continue_button").clicked()
@@ -874,9 +876,9 @@ class T(unittest.TestCase):
         def ack_error():
             # wait until error dialog gets visible
             if Gtk.events_pending():
-                return True
+                return True  # pragma: no cover
             if not self.app.md:
-                return True
+                return True  # pragma: no cover
             self.error_title = self.app.md.get_title()
             self.error_text = self.app.md.get_property("text")
             self.app.md.response(0)
@@ -919,9 +921,9 @@ class T(unittest.TestCase):
 
         def cont():
             if Gtk.events_pending():
-                return True
+                return True  # pragma: no cover
             if not self.app.w("continue_button").get_visible():
-                return True
+                return True  # pragma: no cover
             self.app.w("continue_button").clicked()
             return False
 
@@ -959,16 +961,16 @@ class T(unittest.TestCase):
 
         def cont():
             if Gtk.events_pending():
-                return True
+                return True  # pragma: no cover
             if not self.app.w("continue_button").get_visible():
-                return True
+                return True  # pragma: no cover
             self.app.w("continue_button").clicked()
             return False
 
         # remove the crash from setUp() and create a kernel oops
         os.remove(self.app.report_file)
         subprocess.run(
-            [kernel_oops_path],
+            [get_data_directory() / "kernel_oops"],
             check=True,
             input=b"Plasma conduit phase misalignment",
         )
@@ -996,10 +998,10 @@ class T(unittest.TestCase):
 
         def c():
             if Gtk.events_pending():
-                return True
+                return True  # pragma: no cover
             dont_send_button = self.app.w("dont_send_button")
             if not self.has_click_event_connected(dont_send_button):
-                return True
+                return True  # pragma: no cover
             dont_send_button.clicked()
             return False
 
@@ -1018,10 +1020,10 @@ class T(unittest.TestCase):
 
         def c():
             if Gtk.events_pending():
-                return True
+                return True  # pragma: no cover
             dont_send_button = self.app.w("dont_send_button")
             if not self.has_click_event_connected(dont_send_button):
-                return True
+                return True  # pragma: no cover
             dont_send_button.clicked()
             return False
 
@@ -1048,9 +1050,9 @@ class T(unittest.TestCase):
 
         def cont():
             if Gtk.events_pending():
-                return True
+                return True  # pragma: no cover
             if self.app.tree_model.get_iter_first() is None:
-                return True
+                return True  # pragma: no cover
             self.app.w("continue_button").clicked()
             return False
 
@@ -1085,9 +1087,9 @@ class T(unittest.TestCase):
 
         def cont():
             if Gtk.events_pending():
-                return True
+                return True  # pragma: no cover
             if self.app.tree_model.get_iter_first() is None:
-                return True
+                return True  # pragma: no cover
             self.app.w("continue_button").clicked()
             return False
 
@@ -1147,7 +1149,7 @@ class T(unittest.TestCase):
 
         def show_details(data):
             if not self.app.w("show_details").get_visible():
-                return True
+                return True  # pragma: no cover
 
             data["orig_size"] = self.app.w("dialog_crash_new").get_size()
             data["orig_resizable"] = self.app.w(
@@ -1160,7 +1162,7 @@ class T(unittest.TestCase):
         def hide_details(data):
             # wait until data collection is done and tree filled
             if self.app.tree_model.get_iter_first() is None:
-                return True
+                return True  # pragma: no cover
 
             data["detail_size"] = self.app.w("dialog_crash_new").get_size()
             data["detail_resizable"] = self.app.w(
@@ -1173,7 +1175,7 @@ class T(unittest.TestCase):
         def details_hidden(data):
             # wait until data collection is done and tree filled
             if self.app.w("details_scrolledwindow").get_visible():
-                return True
+                return True  # pragma: no cover
 
             data["hidden_size"] = self.app.w("dialog_crash_new").get_size()
             data["hidden_resizable"] = self.app.w(
@@ -1202,9 +1204,9 @@ class T(unittest.TestCase):
 
         def close(response):
             if Gtk.events_pending():
-                return True
+                return True  # pragma: no cover
             if not self.app.md:
-                return True
+                return True  # pragma: no cover
             self.app.md.response(response)
             return False
 
@@ -1236,16 +1238,16 @@ class T(unittest.TestCase):
 
         def show_details():
             if Gtk.events_pending():
-                return True
+                return True  # pragma: no cover
             if not self.app.w("show_details").get_visible():
-                return True
+                return True  # pragma: no cover
             self.app.w("show_details").clicked()
             GLib.timeout_add(self.POLLING_INTERVAL_MS, close)
             return False
 
         def close():
             if Gtk.events_pending():
-                return True
+                return True  # pragma: no cover
             self.app.w("dialog_crash_new").destroy()
             return False
 
@@ -1274,6 +1276,11 @@ class T(unittest.TestCase):
             f'Für Hilfe gehen Sie über <a href="{url}">{url}</a>.',
         )
 
+    def test_ui_update_view_destroyed(self):
+        """Test ui_update_view if the dialog is already destroyed."""
+        self.app.w("details_treeview").destroy()
+        self.app.ui_update_view()
+
     @staticmethod
     def has_click_event_connected(widget):
         signal_id = GObject.signal_lookup("clicked", widget)
