@@ -15,11 +15,12 @@ import tempfile
 import textwrap
 import unittest
 import unittest.mock
+from gettext import gettext as _
 
 try:
     from PyQt5.QtCore import QCoreApplication, QTimer
     from PyQt5.QtGui import QIcon
-    from PyQt5.QtWidgets import QApplication, QTreeWidget
+    from PyQt5.QtWidgets import QApplication, QProgressBar, QTreeWidget
 
     PYQT5_IMPORT_ERROR = None
 except ImportError as error:
@@ -27,16 +28,10 @@ except ImportError as error:
 
 import apport.crashdb_impl.memory
 import apport.report
-from apport import unicode_gettext as _
 from tests.helper import import_module_from_file, wrap_object
-from tests.paths import is_local_source_directory, local_test_environment
+from tests.paths import get_data_directory, local_test_environment
 
-if is_local_source_directory():
-    apport_kde_path = "kde/apport-kde"
-else:
-    apport_kde_path = os.path.join(
-        os.environ.get("APPORT_DATA_DIR", "/usr/share/apport"), "apport-kde"
-    )
+apport_kde_path = get_data_directory("kde") / "apport-kde"
 if not PYQT5_IMPORT_ERROR:
     apport_kde = import_module_from_file(apport_kde_path)
     MainUserInterface = apport_kde.MainUserInterface
@@ -49,14 +44,14 @@ else:
 )
 class T(unittest.TestCase):
     COLLECTING_DIALOG = unittest.mock.call(
-        os.path.dirname(apport_kde_path),
+        str(apport_kde_path.parent),
         "Collecting Problem Information",
         "Collecting problem information",
         "The collected information can be sent to the developers to improve "
         "the application. This might take a few minutes.",
     )
     UPLOADING_DIALOG = unittest.mock.call(
-        os.path.dirname(apport_kde_path),
+        str(apport_kde_path.parent),
         "Uploading Problem Information",
         "Uploading problem information",
         "The collected information is being sent to the bug tracking system. "
@@ -69,7 +64,7 @@ class T(unittest.TestCase):
         os.environ |= local_test_environment()
         os.environ["LANGUAGE"] = "C"
 
-        cls.argv = [apport_kde_path]
+        cls.argv = [str(apport_kde_path)]
         cls.app = QApplication(cls.argv)
         cls.app.applicationName = "apport-kde"
         cls.app.applicationDisplayName = _("Apport")
@@ -130,7 +125,7 @@ class T(unittest.TestCase):
 
         QTimer.singleShot(0, c)
         result = self.app.ui_present_report_details(True)
-        self.assertFalse(result["report"])
+        self.assertFalse(result.report)
 
     def test_kernel_crash_layout(self):
         """Display crash dialog for kernel crash.
@@ -489,7 +484,7 @@ class T(unittest.TestCase):
                         dialog.hide()
                         return
                 # try again
-                QTimer.singleShot(200, hide_dialog)
+                QTimer.singleShot(200, hide_dialog)  # pragma: no cover
 
             QTimer.singleShot(200, hide_dialog)
             answer = self.app.ui_question_choice(
@@ -515,7 +510,7 @@ class T(unittest.TestCase):
                 self.app.dialog.continue_button.click()
                 return
             # try again
-            QTimer.singleShot(1000, cont)
+            QTimer.singleShot(1000, cont)  # pragma: no cover
 
         QTimer.singleShot(1000, cont)
         with wrap_object(
@@ -561,7 +556,7 @@ class T(unittest.TestCase):
                 return
 
             # try again
-            QTimer.singleShot(200, show_details)
+            QTimer.singleShot(200, show_details)  # pragma: no cover
 
         def cont():
             # wait until data collection is done and tree filled
@@ -574,7 +569,7 @@ class T(unittest.TestCase):
                 self.app.dialog.continue_button.click()
                 return
             # try again
-            QTimer.singleShot(200, cont)
+            QTimer.singleShot(200, cont)  # pragma: no cover
 
         QTimer.singleShot(200, show_details)
         with wrap_object(
@@ -616,7 +611,7 @@ class T(unittest.TestCase):
                 self.app.dialog.continue_button.click()
                 return
             # try again
-            QTimer.singleShot(1000, cont)
+            QTimer.singleShot(1000, cont)  # pragma: no cover
 
         QTimer.singleShot(1000, cont)
         self.app.crashdb.options["problem_types"] = ["bug"]
@@ -649,7 +644,7 @@ class T(unittest.TestCase):
                 self.app.dialog.cancel_button.click()
                 return
             # try again
-            QTimer.singleShot(1000, c)
+            QTimer.singleShot(1000, c)  # pragma: no cover
 
         QTimer.singleShot(1000, c)
         self.app.run_report_bug()
@@ -671,7 +666,7 @@ class T(unittest.TestCase):
                 self.app.dialog.cancel_button.click()
                 return
             # try again
-            QTimer.singleShot(1000, c)
+            QTimer.singleShot(1000, c)  # pragma: no cover
 
         QTimer.singleShot(1000, c)
         self.app.run_report_bug()
@@ -696,7 +691,7 @@ class T(unittest.TestCase):
                 self.app.dialog.continue_button.click()
                 return
             # try again
-            QTimer.singleShot(200, cont)
+            QTimer.singleShot(200, cont)  # pragma: no cover
 
         # upload empty report
         crash_id = self.app.crashdb.upload({})
@@ -732,7 +727,7 @@ class T(unittest.TestCase):
                 self.app.dialog.continue_button.click()
                 return
             # try again
-            QTimer.singleShot(200, cont)
+            QTimer.singleShot(200, cont)  # pragma: no cover
 
         # this test assumes that the source package name is not an
         # installed binary package
@@ -776,3 +771,12 @@ class T(unittest.TestCase):
         self.app.ui_present_report_details(False)
         self.assertFalse(self.app.dialog.send_error_report.isVisible())
         self.assertFalse(self.app.dialog.send_error_report.isChecked())
+
+    def test_ui_set_upload_progress(self):
+        self.app.ui_start_upload_progress()
+        try:
+            self.app.ui_set_upload_progress(0.5)
+            progress = self.app.progress.findChild(QProgressBar, "progress")
+            self.assertEqual(progress.value(), 500)
+        finally:
+            self.app.ui_stop_upload_progress()
