@@ -11,6 +11,22 @@ bin_data = b"ABABABABAB\0\0\0Z\x01\x02"
 
 
 class T(unittest.TestCase):
+    def test_add_tags(self):
+        """Test ProblemReport.add_tags()."""
+        report = problem_report.ProblemReport()
+        report.add_tags({"tag1"})
+        self.assertEqual(report["Tags"], "tag1")
+        report.add_tags(["tag2", "next_tag"])
+        self.assertEqual(report["Tags"], "next_tag tag1 tag2")
+
+    def test_add_tag_drop_duplicates(self):
+        """Test ProblemReport.add_tags() dropping duplicates."""
+        report = problem_report.ProblemReport()
+        report.add_tags({"same"})
+        self.assertEqual(report["Tags"], "same")
+        report.add_tags(["same"])
+        self.assertEqual(report["Tags"], "same")
+
     def test_basic_operations(self):
         """Basic creation and operation."""
         pr = problem_report.ProblemReport()
@@ -66,7 +82,7 @@ class T(unittest.TestCase):
         del pr["Date"]
         self.assertEqual(pr.get_timestamp(), None)
 
-    def test_sanity_checks(self):
+    def test_consistency_checks(self):
         """Various error conditions."""
         pr = problem_report.ProblemReport()
         self.assertRaises(ValueError, pr.__setitem__, "a b", "1")
@@ -205,6 +221,52 @@ class T(unittest.TestCase):
         pr.load(io.BytesIO(b"ProblemType: Crash"))
         self.assertEqual(list(pr.keys()), ["ProblemType"])
 
+    def test_load_binary_blob(self):
+        """Throw exception when binary file (e.g. core) is loaded."""
+        report = problem_report.ProblemReport()
+        with io.BytesIO(b"AB\xfc:CD") as report_file:
+            with self.assertRaisesRegex(
+                problem_report.MalformedProblemReport,
+                "codec can't decode byte 0xfc in position 2",
+            ):
+                report.load(report_file)
+
+    def test_load_missing_colon(self):
+        """Throw exception when key-value line misses a colon as separator."""
+        report = problem_report.ProblemReport()
+        with io.BytesIO(b"\n") as report_file:
+            with self.assertRaisesRegex(
+                problem_report.MalformedProblemReport,
+                r"Line '\\n' does not contain a colon",
+            ):
+                report.load(report_file)
+
+    def test_load_invalid_utf8(self):
+        """Throw exception when binary file is invalid UTF-8."""
+        report = problem_report.ProblemReport()
+        with io.BytesIO(b"\x7fELF\x02\x01\xb0j") as report_file:
+            with self.assertRaisesRegex(
+                problem_report.MalformedProblemReport,
+                r"Line '\\x7fELF\\x02\\x01\\\\xb0j' does not contain a colon",
+            ):
+                report.load(report_file)
+
+    def test_load_incorrect_padding(self):
+        """Throw exception when base64 encoded data has incorrect padding."""
+        report = problem_report.ProblemReport()
+        content = (
+            b"CoreDump: base64\n"
+            b" H4sICAAAAAAC/0NvcmVEdW1wAA==\n"
+            b" 7Z0LYFPV/cdP0rQ\n"
+        )
+        with io.BytesIO(content) as report_file:
+            with self.assertRaisesRegex(
+                problem_report.MalformedProblemReport,
+                "^Malformed problem report: Incorrect padding."
+                " Is this a proper .crash text file\\?$",
+            ):
+                report.load(report_file)
+
     def test_write_fileobj(self):
         """Write a report with a pointer to a file-like object."""
         tempbin = io.BytesIO(bin_data)
